package com.sheca.JShcaCciStd;

import java.io.IOException;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.spongycastle.asn1.pkcs.CertificationRequestInfo;
import org.spongycastle.util.encoders.Base64;

import android.content.Context;

import com.ccit.SecureCredential.bean.AlgorithmConstant;
import com.ccit.SecureCredential.bean.ResultVo;
import com.ccit.SecureCredential.sdk.SDKImpl;

import com.sheca.javasafeengine;
import com.sheca.sm2utils.JPkcs10CertReq;
import com.sheca.sm2utils.JSm2Utils;
import com.sheca.sm2utils.shcaP7structure;

public class JShcaCciStd extends javasafeengine{

	private static JShcaCciStd sdk = null;
	private static SDKImpl sdkImpl;
	private static String gAppID;
	private static String gDevID;
	
	private static final int K_SDK_ERROR = -2;
	@SuppressWarnings("unused")
	private static final int K_PARAM_ERROR = -3;
		
	public static JShcaCciStd getIntence(Context paramContext){
		if (sdk == null){
			sdk = new JShcaCciStd();
			sdk.setContext(paramContext);
			sdkImpl = SDKImpl.getIntence( paramContext );
			return sdk;	
		}
		return sdk;
	}
	
	private JShcaCciStd(){
		sdkImpl = null;
		gAppID = null;
		gDevID = null;
	}
	
	private boolean checkStrIsNotNull(String d){
		boolean ret = false;
		if (d != null){
			if (d.length() > 0){
				ret = true;
			}
		}
		return ret;
	}
	
	@SuppressWarnings("unused")
	private boolean checkBytesIsNotNull(byte[] d){
		boolean ret = false;
		if (d != null){
			if (d.length > 0){
				ret = true;
			}
		}
		return ret;		
	}
	
	private String MD5UserPin(String pin){		
		String ret = null;
		if (checkStrIsNotNull(pin)){
			byte[] r = this.digest(pin.getBytes(), "MD5", "");
			ret = new String(Base64.encode(r));			
		}
		return ret;
	}

	@SuppressWarnings("unused")
	private String verifyUserPinAndOutMd5Pin(String containerID,String pin){
		String md5Pin = null;
		if (sdkImpl != null){
			String md5PinTmp = MD5UserPin(pin);
			if (md5PinTmp != null){
				ResultVo ret = sdkImpl.checkPin(md5PinTmp, containerID);
				if (ret != null){
					if (ret.getResultCode() == 0){
						md5Pin = md5PinTmp;
					}
				}
			}
		}		
		return ( md5Pin );
	}
	
	public int getLastErrCode(){
		int retcode = K_SDK_ERROR;
		if (sdkImpl != null){
			retcode = sdkImpl.getLastErrorCode();
		}
		return retcode;
	}
	
	/**
	 * SDKгʼ
	 * @param ctx
	 * @param appID
	 * @param isBindSimCard
	 * @param serviceURL
	 * @param timeOut
	 * @param testFlag
	 * @return
	 */
	public int initService(String appID,boolean isBindSimCard,String serviceURL,int timeOut,boolean testFlag ){
		int retcode = -1;
		ResultVo rVo = sdkImpl.initSDK(appID, isBindSimCard, serviceURL, timeOut);
		if(rVo != null){
			retcode = rVo.getResultCode();
			if(retcode == 0){				
				gAppID = appID;
				gDevID = rVo.getCipherDeviceId();
			}
		}
		return retcode;
	}
	
	/**
	 * SM2ԿԣعԿϢ
	 * @param pin
	 * @return
	 */
	public shcaCciStdGenKeyPairRes genSM2KeyPair(String pin){
		//
		shcaCciStdGenKeyPairRes r = null;
		ResultVo rVo = null;		
		//genKeyPair
		rVo = sdkImpl.generateKeyPair(gAppID, AlgorithmConstant.ASY_SM2, pin, gDevID, "");
		if(rVo != null){
			r = new shcaCciStdGenKeyPairRes();
			r.retcode = rVo.getResultCode();
			if(rVo.getResultCode() == 0){								
				r.containerID = rVo.getContainerId();
				r.pubkey = rVo.getPublicKey();
			}
		}
		return r;
	}
	
	/**
	 * RSAԿ,عԿϢ
	 * @param pin
	 * @return
	 */
	public shcaCciStdGenKeyPairRes genRSAKeyPair(String pin){
		//
		shcaCciStdGenKeyPairRes r = null;
		ResultVo rVo = null;		
		//genKeyPair
		rVo = sdkImpl.generateKeyPair(gAppID, AlgorithmConstant.ASY_RSA, pin, gDevID, "");
		if(rVo != null){
			r = new shcaCciStdGenKeyPairRes();
			r.retcode = rVo.getResultCode();
			if(rVo.getResultCode() == 0){								
				r.containerID = rVo.getContainerId();
				r.pubkey = rVo.getPublicKey();
			}
		}
		return r;		
	}
	
	/**
	 * DNSM2Կ֤Ϣ
	 * @param dn
	 * @param bPubkey
	 * @param pin
	 * @param containerID
	 * @return
	 * @throws IOException
	 */
	public String getSM2PKCS10(String dn,byte[] bPubkey,String pin,String containerID) throws IOException{
		String strP10 = null;
		//
		JPkcs10CertReq p10req = new JPkcs10CertReq();
		//
		CertificationRequestInfo reqInfo = null;		
		reqInfo = p10req.makeSM2CertReqInfo(dn, bPubkey);
		//reqInfo sign(sm3withSm2)
		byte[] signdata = sdkImpl.signature(AlgorithmConstant.ASY_SM2, reqInfo.getEncoded(), pin, containerID);
		// create pkcs10 request data.
		strP10 = p10req.makeSM2Pkcs10String(reqInfo,signdata);
		//
		return strP10;
	}
	
	/**
	 * SMǩ֤鼰֤
	 * @param containerID
	 * @param pin
	 * @param signatureCert
	 * @param encryptCert
	 * @param encryptPrikeyCipher
	 * @return
	 */
	public int saveSM2DoubleCert(String containerID,String pin,String signatureCert,String encryptCert,String encryptPrikeyCipher){
		int retcode = sdkImpl.importCertInfo_SH(gAppID, AlgorithmConstant.ASY_SM2, gDevID, containerID, pin, signatureCert, encryptCert, encryptPrikeyCipher, AlgorithmConstant.SYM_SM1);
		return retcode;
	}
	
	/**
	 * SM2ǩ֤
	 * @param containerID
	 * @param pin
	 * @param signatureCert
	 * @return
	 */
	public int saveSM2SignatureCert(String containerID,String pin,String signatureCert){
		int retcode = sdkImpl.importCertInfo_SH(gAppID, AlgorithmConstant.ASY_SM2, gDevID, containerID, pin, signatureCert, "", "", AlgorithmConstant.SYM_SM1);
		return retcode;
	}
		
	/**
	 * IDSM2ǩ֤
	 * @param containerID
	 * @return
	 */
	public String readSM2SignatureCert(String containerID){
		return (sdkImpl.getCert(gAppID, 0, containerID));
	}
	
	/**
	 * IDSM2֤
	 * @param containerID
	 * @return
	 */
	public String readSM2EncryptCert(String containerID){
		return (sdkImpl.getCert(gAppID, 1, containerID));
	}
	
	/**
	 * ȡIDб
	 * @return
	 */
	public List<String> enumAllContianer(){
		List<String> lstRet = null;		
		List<Map<String, String>> lstObj = null;
		ResultVo rVo = sdkImpl.getContainerIdList(gAppID, gDevID);
		if(rVo != null){
			if (rVo.getResultCode() == 0){
				lstObj = rVo.getContainerIdList();
				if(lstObj != null){
					lstRet = new ArrayList<String>();
					for (Map<String, String> m : lstObj) {  
						for (String k : m.keySet()) {
							lstRet.add(k);
						}			  
				    }
				}
			}
		}
		return lstRet;
	}
	
	/**
	 * ݽSM2ǩ
	 * @param oridata
	 * @param pin
	 * @param containerID
	 * @return
	 */
	public byte[] doSM2Signature(byte[] oridata,String pin,String containerID){
		byte[] ans1_sign = null;
		byte[] ret = sdkImpl.signature(AlgorithmConstant.ASY_SM2, oridata, pin, containerID);		
		if (ret != null){
			ans1_sign = JSm2Utils.toAsn1EncodeWithSM2Sign(ret);
		}
		return ans1_sign;
	}
	
	/**
	 * 
	 * @param cert
	 * @param oriData
	 * @return
	 */
	public byte[] doSM2AsyEncrypt(String cert,byte[] oriData){
		byte[] ret = null;
		ret = sdkImpl.asyEncrypt(AlgorithmConstant.ASY_SM2, cert, oriData);
		return ret;
	}
	
	/**
	 * 
	 * @param containerID
	 * @param pin
	 * @param encryptData
	 * @return
	 */
	public byte[] doSM2AsyDecrypt(String containerID,String pin,byte[] encryptData){
		byte[] ret = null;
		ret = sdkImpl.asyDecrypt(AlgorithmConstant.ASY_SM2, encryptData, pin, containerID);
		return ret;
	}
	
	/**
	 * ԳƼ
	 * @param algoID
	 * @param symKey
	 * @param inData
	 * @return
	 */
	public byte[] doSM2SymEncrypt(int algoID,byte[] symKey,byte[] inData){
		byte[] r = null;
		r = sdkImpl.symEncrypt(algoID, symKey, inData);		
		return r;
	}
	
	/**
	 * Գƽ
	 * @param algoID
	 * @param symKey
	 * @param encData
	 * @return
	 */
	public byte[] doSM2SymDecrypt(int algoID,byte[] symKey,byte[] encData){
		byte[] r = null;
		r = sdkImpl.symDecrypt(algoID, symKey, encData);
		return r;		
	}
	
	/**
	 * ֤ûPIN
	 * @param containerID
	 * @param pin
	 * @return
	 */
	public int verifyUserPin(String containerID,String pin){
		ResultVo ret = sdkImpl.checkPin(pin, containerID);
		return (ret.getResultCode());
	}
	
	/**
	 * ID޸pinӿڣӿڴpin֤
	 * @param containerID
	 * @param oldPin
	 * @param newPin
	 * @return
	 */
	public int changePin(String containerID,String oldPin,String newPin){
		//ResultVo ret = sdkImpl.modPin(oldPin,newPin, containerID);
		ResultVo ret = sdkImpl.modPin(oldPin,newPin, containerID, 0);
		return (ret.getResultCode());
	}
	
	/**
	 * ûPIN
	 * @param containerID
	 * @return
	 */
	public int unLockPin(String containerID){		
		return (sdkImpl.UnLockPin(containerID));
	}

	/**
	 * SM2еԿ֤
	 * @param containerID
	 * @return
	 */
	public int detroySM2Container(String containerID){
		return (sdkImpl.delCertByCId(gAppID, containerID));
	}
	
	/**
	 * 
	 * @param oridata
	 * @param symAlgId
	 * @param certB64
	 * @return
	 */
	private shcaP7structure generateKeyWithEPK_SM2(byte[] oridata,int symAlgId,String certB64){
		shcaP7structure p7 = new shcaP7structure();
		//
		byte[] bRan = this.random(16, "SHA1PRNG", "SUN");
		//
		p7.cert = Base64.decode(certB64);		
		//
		p7.aymAlgoID = 1;//sm2
		byte[] tmp_symkeyCipher = this.doSM2AsyEncrypt(certB64, bRan);		
		p7.symkeyCipher = JSm2Utils.toAns1EncodeWithSM2Cipher(tmp_symkeyCipher);
		//
		p7.symAlgoID = 11;
		p7.cipherText = this.encrypt(oridata, bRan, "SM4", "ShecaJCE");
		p7.retcode = 0;
		return p7;
	}
	
	/**
	 * 
	 * @param oridata
	 * @param certB64
	 * @return
	 */
	public byte[] doEncSM2Enveloper(byte[] oridata,String certB64){
		byte[] bRet = null;
		shcaP7structure p7 = this.generateKeyWithEPK_SM2(oridata,11,certB64);
		bRet = this.encodeEnveloper(p7);
		return bRet;
	}
	
	/**
	 * SM2ŷ
	 * @param containerID
	 * @param pin
	 * @param p7data
	 * @return
	 */
	public byte[] doDecSM2Enveloper(String containerID,String pin,byte[] p7data){
		byte[] bRet = null;
		shcaP7structure p7 = this.decodeEnveloper(p7data);
		byte[] tmp_symkeyCipher = JSm2Utils.toAsn1DecodeWithSM2Cipher( p7.symkeyCipher );
		byte[] symKey = doSM2AsyDecrypt(containerID,pin,tmp_symkeyCipher);
		if (symKey != null){
			bRet = this.decrypt(p7.cipherText, symKey, "SM4", "ShecaJCE");
		}
		return bRet;
	}
	/**
	 * SDKеǩӿ֤ǩ 
	 * @param cert
	 * @param oridata
	 * @param signdata
	 * @return
	 */
	public int verifySM2SignatureBySdk(byte[] oridata,byte[] signdata,String cert){
		int retcode = -1;
		retcode = sdkImpl.verifySignature(AlgorithmConstant.ASY_SM2, cert, oridata, signdata);
		return retcode;
	}
	
	/**
	 * SDKеǩӿ֤֤ЧԼ֤
	 * @param cert
	 * @param certchain
	 * @return
	 */
	public int verifySM2CertBySdk(String cert,String certchain){
		int retcode = JSm2Utils.SE_ECC_ERROR_VERIFY_CERT_CHAIN;	
		X509Certificate oCert = JSm2Utils.getX509CertFromBuffer(cert);
		try {
			oCert.checkValidity();
			//
			byte[] msg = oCert.getTBSCertificate();
			byte[] signData = oCert.getSignature();
			String strIssueCert = null;
			List<String> lst = JSm2Utils.getCertsFormCertChain( certchain );
			int i=0;
			for(i=0;i<lst.size()-1;i++){
				strIssueCert = lst.get(i);			
				if (verifySM2SignatureBySdk(msg, signData,strIssueCert) == 0){
					retcode = 0;
					oCert = JSm2Utils.getX509CertFromBuffer( strIssueCert );
					msg = oCert.getTBSCertificate();
					signData = oCert.getSignature();
				}else{
					retcode = JSm2Utils.SE_ECC_ERROR_VERIFY_CERT_CHAIN;
					break; 
				}					
			}
			//check self.
			if( (retcode==0) && (i==lst.size()-1)){
				strIssueCert = lst.get(i);
				oCert = JSm2Utils.getX509CertFromBuffer( strIssueCert );
				msg = oCert.getTBSCertificate();
				signData = oCert.getSignature();
				retcode = (verifySM2SignatureBySdk(msg, signData,strIssueCert)==0) ?  0 : JSm2Utils.SE_ECC_ERROR_VERIFY_CERT_CHAIN;			
			}			
		} catch (CertificateExpiredException e) {
			retcode = JSm2Utils.SE_ECC_ERROR_CERT_EXPIRED;
		} catch (CertificateNotYetValidException e) {
			retcode = JSm2Utils.SE_ECC_ERROR_INVALID_CERT;
		} catch (CertificateEncodingException e) {
			retcode = JSm2Utils.SE_ECC_ERROR_INVALID_CERT;
		} catch (CertificateException e) {
			retcode = JSm2Utils.SE_ECC_ERROR_INVALID_SPC;			
		}
		return retcode;
	}
}
